﻿using CMS.Base;
using CMS.CustomTables;
using CMS.DataEngine;
using CMS.EventLog;
using CMS.Helpers;
using CMS.Membership;
using CMS.SiteProvider;
using CMS.Synchronization;
using System;
using System.Data;

[ContentStagingLogging]
public partial class CMSModuleLoader
{

    private class ContentStagingLogging : CMSLoaderAttribute
    {
        public override void Init()
        {
            StagingEvents.SynchronizeTask.After += SynchronizeTask_After;
            StagingEvents.SynchronizeTask.Failure += SynchronizeTask_Failure;
        }

        private void SynchronizeTask_After(object sender, StagingTaskEventArgs e)
        {
            LogEvent(e.Task, "Success");
        }

        private void SynchronizeTask_Failure(object sender, StagingTaskEventArgs e)
        {
            LogEvent(e.Task, "Failure");
        }

        private void LogEvent(StagingTaskInfo task, string eventType)
        {
            try
            {
                // Prepares the code name (class name) of the custom table to which the data record will be added
                string customTableClassName = "custom.contentstaginglog";

                // Gets the custom table
                DataClassInfo customTable = DataClassInfoProvider.GetDataClassInfo(customTableClassName);
                if (customTable != null)
                {
                    // Creates a new custom table item 
                    CustomTableItem item = CustomTableItem.New(customTableClassName);
                    QueryDataParameters parameters = new QueryDataParameters();
                    parameters.Add("TaskID", task.TaskID);
                    item.SetValue("Username", MembershipContext.AuthenticatedUser.UserName);
                    item.SetValue("TaskTitle", task.TaskTitle);
                    item.SetValue("TaskTime", task.TaskTime);
                    item.SetValue("SynchronizationTime", DateTime.Now);
                    item.SetValue("SiteName", SiteContext.CurrentSiteName);
                    item.SetValue("EventType", eventType);
                    item.SetValue("TaskData", task.TaskData);
                    //Check if the task failed
                    if (eventType == "Failure")
                    {
                        //get the error message
                        const string SELECT_ERROR_FOR_TASK = @"SELECT TOP 1 SynchronizationErrorMessage FROM Staging_Synchronization WHERE SynchronizationTaskID = @TaskID ORDER BY SynchronizationID DESC";
                        DataSet dsError = ConnectionHelper.ExecuteQuery(SELECT_ERROR_FOR_TASK, parameters, QueryTypeEnum.SQLQuery, false);
                        if (!DataHelper.IsEmpty(dsError))
                        {
                            string errorMessage = dsError.Tables[0].Rows[0][0] as string;
                            item.SetValue("ErrorMessage", errorMessage);
                        }
                    }

                    // Save the new custom table record into the database without any logging
                    using (CMSActionContext context = new CMSActionContext())
                    {
                        context.LogSynchronization = false;
                        context.LogExport = false;
                        context.LogIntegration = false;
                        context.LogEvents = false;
                        item.Insert();
                    }
                }
            }
            catch (Exception e)
            {
                EventLogProvider.LogException("ContentStagingLogging", "LogEvent", e);
            }
        }
    }
}
